package klassen;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

public class Buchliste {

	private List<Buch> liste = new ArrayList<Buch>(); 
	NumberFormat nf = NumberFormat.getInstance(Locale.GERMAN);
	
	public Buchliste() throws IOException, ParseException
	{
		Path path = Paths.get("C:\\Users\\post-\\OneDrive\\Dokumente\\java Tutorial Buchladen\\liste.csv");
		
		List<String> zeilen = Files.readAllLines(path);
		
		for(String zeile: zeilen)
		{
			String[] werte = zeile.split(";");
			
			//0: nummer, 1: titel
			double pr = (double) nf.parse(werte[2]);
			int menge = Integer.parseInt(werte[3]);
			
			Buch.Art art = Buch.Art.valueOf(werte[4]);
			
			liste.add(new Buch(werte[0], werte[1], pr, menge, art));
		}
	}
	
	public Buch[] getAll()
	{
		return liste.toArray(new Buch[] {});
	}
	
	//Filtern nach Kategorie
	public Buch[] getAll(Buch.Art art)
	{
		ArrayList<Buch> suche = liste.stream().filter(b -> b.kategorie == art).collect(Collectors.toCollection(ArrayList::new));
		return suche.toArray(new Buch[] {});
	}
	
	//Umsatzberechnung
	public double getUmsatz()
	{
		return liste.stream().mapToDouble(b -> b.preis * b.anzahl).sum();
	}
	
	public double getUmsatz(Buch.Art art)
	{
		return liste.stream().filter(b -> b.kategorie == art).mapToDouble(b -> b.preis * b.anzahl).sum();
	}
	
	//Sortieren nach nummer, preis, kategorie 
	
	public Buch[] sortieren(String column)
	{
		ArrayList<Buch> suche; 
		
		switch(column)
		{
		case "nummer": suche = liste.stream().sorted((b1,b2) -> b1.nummer.compareTo(b2.nummer)).collect(Collectors.toCollection(ArrayList::new));break;
		case "preis": suche = liste.stream().sorted((b1,b2) -> (int)(b2.preis - b1.preis)).collect(Collectors.toCollection(ArrayList::new));break;
		case "kategorie": suche = liste.stream().sorted((b1,b2) -> b1.kategorie.compareTo(b2.kategorie)).collect(Collectors.toCollection(ArrayList::new));break;
		default: suche = new ArrayList<>();
		}
		
		return suche.toArray(new Buch[] {});
	}
	
	public Optional<Buch> suchen(String nummer)
	{
		return liste.stream().filter(b -> b.nummer.equals(nummer)).findFirst();
	}
	
	public boolean eintragen(String nummer, String titel, double preis, int anzahl, Buch.Art art)
	{
		try
		{
			Buch b = new Buch(nummer, titel, preis, anzahl, art);
			
			String out = nummer + ";" + titel + ";" + nf.format(preis) + ";" + anzahl + ";" + art + "\n";
			
			Path path = Paths.get("C:\\Users\\post-\\OneDrive\\Dokumente\\java Tutorial Buchladen\\liste.csv");
			
			Files.writeString(path, out, StandardOpenOption.APPEND);
			liste.add(b);
		}
		catch(Exception e)
		{
			return false;
		}
		
		return true; 
	}
}



